var backWidth = 220;
var backHeight = 290;
var frontWidth = 320;
var frontHeight = 200;
var backShown = false;
var interval = null;

var city = "Home";
var cityLat = 0;
var cityLong = -getTimezoneOffset(new Date()) / 12 / 60 * 180;

var timeZone = 0;
var dst = false;
var show12Clock = false;
var showSeconds = true;

var daysAfterMonth = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334];
var daysInMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
var specials = [-9.5, -3.5, 3.5, 4.5, 5.5, 5.75, 6.5, 8.75, 9.5, 10.5, 11.5, 12.75];

function setup() {
  if (window.widget) {
    if (!widget.context) {
      document.getElementById("Widgetop").style.display = "block";
    }
    window.resizeTo(frontWidth, frontHeight);  
    widgetSetup();
    fillTimeZones();
    widget.onshow = readPrefs;
    widget.onhide = onhide;
    widget.onremove = clearPrefs;
    document.getElementById("MapImage").onclick = mapClicked;
    document.getElementById("Map").onclick = closeMap;
    
    var date = new Date();
    if (navigator.language) {
      show12Clock = navigator.language.toLowerCase().indexOf("en") >= 0 
        || date.toLocaleTimeString().toLowerCase().indexOf("am") >= 0
        || date.toLocaleTimeString().toLowerCase().indexOf("pm") >= 0;
    }
    DWRUtil.addOptions("MonthSelect", ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]);
    $("MonthSelect").selectedIndex = date.getMonth();
    fillDaySelect();
    $("DaySelect").selectedIndex = date.getDate() - 1;
    $("DstInput").checked = dst;
    dateUpdated();        
  }  
}

function readPrefs() {
  var value = widget.preferenceForKey(widget.identifier + "-location");
  if (value != null) {
    city = value;    
    if (widget.mobile) {
      if (city.length > 0) {
        widget.mobile.setTitle(city);
      }
      else {
        widget.mobile.setTitle("World Clock");      
      }  
    }
  }
  document.getElementById("Location").innerHTML = city;
  document.getElementById("LocationInput").value = city;

  value = widget.preferenceForKey(widget.identifier + "-12clock");
  if (value != null) {
    show12Clock = value == "true";
  }
  
  value = widget.preferenceForKey(widget.identifier + "-zone");
  if (value != null) {
    timeZone = parseFloat(value);    
  }
  else {
    timeZone = -(getTimezoneOffset(new Date())) / 60;
  }
  document.getElementById("TimeZone").value = timeZone;

  value = widget.preferenceForKey(widget.identifier + "-dst");
  if (value != null) {
    dst = value == "true";    
  }
  else {
    dst = false;
  }
  document.getElementById("DstInput").checked = dst;

  value = widget.preferenceForKey(widget.identifier + "-lat");
  if (value != null) {
    cityLat = parseFloat(value);    
  }
  value = widget.preferenceForKey(widget.identifier + "-long");
  if (value != null) {
    cityLong = parseFloat(value);    
  }
  document.getElementById("LatInput").value = cityLat;
  document.getElementById("LongInput").value = cityLong;
  update();
}

function clearPrefs() {
  widget.setPreferenceForKey(null, widget.identifier + "-location");
  widget.setPreferenceForKey(null, widget.identifier + "-12clock");
  widget.setPreferenceForKey(null, widget.identifier + "-zone");
  widget.setPreferenceForKey(null, widget.identifier + "-lat");
  widget.setPreferenceForKey(null, widget.identifier + "-long");
  widget.setPreferenceForKey(null, widget.identifier + "-dst");
}

function onhide() {
  if (interval) {
    clearInterval(interval);
  }  
}

  function today() {
    var date = new Date();
    updateDate(date);
  }
  
  function tomorrow() {
    var date = new Date();
    date.setDate(date.getDate() + 1);
    updateDate(date);
  }
  
  function updateDate(date) {
    month = date.getMonth();
    day = date.getDate() - 1;
    $("MonthSelect").selectedIndex = month;
    fillDaySelect();
    $("DaySelect").selectedIndex = day;
    update();    
  }
  
  function dateUpdated() {
    month = $("MonthSelect").selectedIndex;
    day = $("DaySelect").selectedIndex;
    fillDaySelect();
    update();    
  }
  
  function update() {
    var zone = parseFloat($("TimeZone").value);
    timeZone = zone;
    if ($("DstInput").checked) {
      zone += 1;
    }
    dst = $("DstInput").checked;
    calcSun(daysAfterMonth[$("MonthSelect").selectedIndex] + $("DaySelect").selectedIndex, 
            zone, cityLat, cityLong);      
  }
  
  function calcSun(day, zone, lat, lng) {
    // Formula from http://mathforum.org/library/drmath/view/56478.html
    var test = 0.39795 * Math.cos(0.2163108 + 2*Math.atan(0.9671396 * Math.tan(0.00860 * (day-186))));
    var p = Math.asin(test);
    
    var nom = Math.sin(0.83333*Math.PI/180) + Math.sin(lat*Math.PI/180)*Math.sin(p);
    var denom = Math.cos(lat*Math.PI/180)* Math.cos(p);
  
    test = nom/denom;
    
    if (Math.abs(test) > 1) {
      if (test > 0) {
        this.sunrise = -60;
        this.sunset = 25 * 60; //never
      }
      else {
        this.sunrise = 25 * 60;
        this.sunset = -60; //never      
      }  
      return;
    }
  
    var d = 24 - (24/Math.PI) * Math.acos(test);
    $("SunTotalHours").innerHTML = Math.floor(d);
    $("SunTotalMinutes").innerHTML = Math.floor((d - Math.floor(d)) * 60);

    var shift = (- lng) / 360.0 * 24 + 12;
    //alert((shift - d / 2) + " " + (shift + d / 2));
    sunrise = (shift - d / 2) * 60;
    sunset = (shift + d / 2) * 60;
    if (sunrise < 0) {
      $("SunriseGMT").innerHTML = "";
      $("SunsetGMT").innerHTML = "No Sunset";
    }
    else if (sunset < 0) {
      $("SunriseGMT").innerHTML = "No Sunrise";
      $("SunsetGMT").innerHTML = "";
    }
    else {
      $("SunriseGMT").innerHTML = "\u2191 " + this.formatTime2(parseInt(this.sunrise / 60), 
                                     parseInt(this.sunrise % 60), 
                                     parseInt((this.sunrise - Math.floor(this.sunrise)) * 60)); 
      $("SunsetGMT").innerHTML = "\u2193 " + this.formatTime2(parseInt(this.sunset / 60), 
                                     parseInt(this.sunset % 60), 
                                     parseInt((this.sunset - Math.floor(this.sunset)) * 60)); 
    }                                 
    
    var shift = (zone*15 - lng) / 360.0 * 24 + 12;
    //alert((shift - d / 2) + " " + (shift + d / 2));
    sunrise = (shift - d / 2) * 60;
    sunset = (shift + d / 2) * 60;
    if (this.sunrise < 0) {
      $("SunriseLocal").innerHTML = "";
      $("SunsetLocal").innerHTML = "No Sunset";
    }
    else if (this.sunset < 0) {
      $("SunriseLocal").innerHTML = "No Sunrise";
      $("SunsetLocal").innerHTML = "";
    }
    else {
      $("SunriseLocal").innerHTML = "\u2191 " + this.formatTime2(parseInt(this.sunrise / 60), 
                                     parseInt(this.sunrise % 60), 
                                     parseInt((this.sunrise - Math.floor(this.sunrise)) * 60)); 
      $("SunsetLocal").innerHTML = "\u2193 " + this.formatTime2(parseInt(this.sunset / 60), 
                                     parseInt(this.sunset % 60), 
                                     parseInt((this.sunset - Math.floor(this.sunset)) * 60)); 
    }                                 
  }

  function formatTime(date) {
    var hours = date.getHours();
    var minutes = date.getMinutes() + "";
    var seconds = lastSeconds % 60 + "";
    return this.formatTime2(date.getHours(), date.getMinutes(), parseInt(date.getTime() / 1000 % 60));
  }

  function formatTime2(hours, minutes, seconds) {
    var isPM = hours >= 12;
    if (this.show12Clock) {
      hours = hours % 12;
      if (hours == 0) {
        hours = 12;
      }
    }
    hours = hours + "";
    if (hours.length < 2 && !this.show12Clock) {
      hours = "0" + hours;
    }    
    minutes = minutes + "";
    if (minutes.length < 2) {
      minutes = "0" + minutes;
    }
    seconds = seconds + "";
    if (seconds.length < 2) {
      seconds = "0" + seconds;
    }
    var result = hours + ":" + minutes + ":" + seconds;
    if (this.show12Clock) {
      if (isPM) {
        result += "<span class='timeExt'>PM</span>";
      }
      else {
        result += "<span class='timeExt'>AM</span>";
      }
    }
    return result;
  }
  
  function fillTimeZones() {
    var select = document.getElementById("TimeZone");
    for (var i = -13; i < 14; i++) {
    	var option = document.createElement("option");
    	option.value = i;
    	if (i < 0) {
    	  option.innerHTML = "GMT " + i + ":00";
    	}
    	else {
    	  option.innerHTML = "GMT +" + i + ":00";  	  
    	}
    	select.appendChild(option);  
    }
    for (var i = specials.length - 1; i >= 0; i--) {
    	var option = document.createElement("option");
    	var value = specials[i];
    	option.value = value;
    	var min = Math.abs(parseInt((value - parseInt(value)) * 60));
    	if (value < 0) {
    	  option.innerHTML = "GMT " + parseInt(value) + ":" + min;
    	}
    	else {
    	  option.innerHTML = "GMT +" + parseInt(value) + ":" + min;  	  
    	}
    	var pre = select.options[13 + parseInt(value) + 1];
      select.insertBefore(option, pre);
    }
    select.value = timeZone;
  }

  function getTimezoneOffset(date) {
    var offset = date.getTimezoneOffset();
    var diff = -(date.getHours() - date.getUTCHours()) * 60;
    if (Math.abs(diff) != Math.abs(offset)) {
      if (diff > 0) {
        diff -= 24 * 60;
      }
      else {
        diff += 24 * 60;
      }  
    }
    if (diff != offset) {
      offset = -offset;
    }
    return offset;
  }
  
  function fillDaySelect() {
    var month = $("MonthSelect").selectedIndex;
    var select = $("DaySelect");
    var day = select.selectedIndex;
    DWRUtil.removeAllOptions(select);
    for (var i = 1; i <= daysInMonth[month]; i++) {
    	var option = document.createElement("option");
    	option.value = i;
    	option.innerHTML = i;  	  
    	select.appendChild(option);  
    }
    if (day < select.length) {
      select.selectedIndex = day;
    }
    else {
      select.selectedIndex = select.length - 1;
    }
  }
  
  function dstChanged() {
    dst = document.getElementById("DstInput").checked;
    widget.setPreferenceForKey("" + dst, widget.identifier + "-dst");
    update();
  }
  
function parseLatLong() {
  try {
    var lat = parseFloat(document.getElementById("LatInput").value);
    if (lat > -90 && lat < 90) {
      cityLat = lat;
      widget.setPreferenceForKey(cityLat, widget.identifier + "-lat");
    }
  }
  catch (e) {
    document.getElementById("LatInput").value = cityLat;
  }  
  try {
    var lng = parseFloat(document.getElementById("LongInput").value);
    if (lng > -180 && lng < 180) {
      cityLong = lng;
      widget.setPreferenceForKey(cityLong, widget.identifier + "-long");
    }
  }
  catch (e) {
    document.getElementById("LongInput").value = cityLong;    
  }  
}

function getTimezoneOffset(date) {
  var offset = date.getTimezoneOffset();
  var diff = -(date.getHours() - date.getUTCHours()) * 60;
  if (Math.abs(diff) != Math.abs(offset)) {
    if (diff > 0) {
      diff -= 24 * 60;
    }
    else {
      diff += 24 * 60;
    }  
  }
  if (diff != offset) {
    offset = -offset;
  }
  return offset;
}

function toggleTimeFormat() {
  show12Clock = !show12Clock;
  widget.setPreferenceForKey("" + show12Clock, widget.identifier + "-12clock");
  update();
}

function showMap() {
  document.getElementById("Map").style.display = "block";
}

function closeMap() {
  document.getElementById("Map").style.display = "none";
}

function mapClicked(event) {
  if (window.event) {
    event = window.event;
  }
  var img = document.getElementById("MapImage");
  var x = img.width - (backWidth - event.clientX - 10);
  var y = img.height - (backHeight - event.clientY - 30);
  document.getElementById("LongInput").value = ((x / img.width - 0.5) * 360).toFixed(1);
  document.getElementById("LatInput").value = (-(y / img.height - 0.5) * 180).toFixed(1);
  closeMap();
}

function widgetSetup() {
  if (window.widget) {
  	var gDoneButton = new AppleGlassButton(document.getElementById("DoneButton"), 
  	                                      "Done", hideBack); 
  	var gDemoButton = new AppleGlassButton(document.getElementById("MapButton"), 
  	                                      "Map", showMap); 
	  var gInfoButton = new AppleInfoButton(document.getElementById("InfoButton"), 
	                                        document.getElementById("front"), 
	                                        "white", "black", showBack);
	  gInfoButton.setCircleOpacity(0.5);                                      
  }
}

function showBack() {
    var front = document.getElementById("front");
    var back = document.getElementById("back");
 
 
    backShown = true;
    if (window.innerWidth) {
      frontWidth = window.innerWidth;
      frontHeight = window.innerHeight;
    }
    else {
      frontWidth = document.documentElement.clientWidth;
      frontHeight = document.documentElement.clientHeight;
    }  
    window.resizeTo(backWidth, backHeight);
 
    if(window.widget)
        widget.prepareForTransition("ToBack");
    front.style.display="none";
    back.style.display="block";
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);  
}

function hideBack() {
    window.resizeTo(frontWidth, frontHeight);  
    var front = document.getElementById("front");
    var back = document.getElementById("back");
 
    if (window.widget)
        widget.prepareForTransition("ToFront");
 
    city = document.getElementById("LocationInput").value;
    document.getElementById("Location").innerHTML = city;
    widget.setPreferenceForKey(city, widget.identifier + "-location");
    if (widget.mobile) {
      if (city.length > 0) {
        widget.mobile.setTitle(city);
      }
      else {
        widget.mobile.setTitle("Sunset");      
      }  
    }
        
    timeZone = parseFloat(document.getElementById("TimeZone").value);
    widget.setPreferenceForKey(timeZone, widget.identifier + "-zone");
    
    dst = document.getElementById("DstInput").checked;
    widget.setPreferenceForKey("" + dst, widget.identifier + "-dst");
    
    parseLatLong();
    update();
    
    backShown = false;
    back.style.display="none";
    front.style.display="block";
 
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);
}                                                
  